<?php
namespace Controllers;

use \Zeedhi\Framework\DTO;
use \Zeedhi\Framework\Report\ReportService;
use \Zeedhi\Framework\DTO\Response\Error;
use \Zeedhi\Framework\DataSource\FilterCriteria;
use \Zeedhi\Framework\DTO\Request\Row;
use \Zeedhi\Framework\DB\StoredProcedure\StoredProcedure;
use Zeedhi\Framework\DB\StoredProcedure\Param;

class HistoricoReceitas extends \Zeedhi\Framework\Controller\Report {

  protected $environment;
  protected $birtStrategy;
  protected $connection;

  public function __construct(ReportService $reportService, $environment, $connection, $birtStrategy) {
      $this->birtStrategy = $birtStrategy;
      $this->environment = $environment;
      $this->connection = $connection;
      parent::__construct($reportService);
  }

  protected $reportMapping = array (
    'COC03500' =>
    array (
      'strategy' => 'BIRT',
      'parameterMapping' =>
      array (
        "CDFILIAL"          => "CDFILIAL",
        "CDCIA"             => "CDCIA",
        "CDMANCIA"          => "CDMANCIA",
        "IDIOMA"            => "IDIOMA",
        "DTCUSTOAP"         => "DATACUSTO",
        "CDSUBGRU"          => "CDSUBGRU",
        "INFLACAO"          => "INFLACAO",
        "NMTIPOCUSTO"       => "IDTIPOCUSTO",
        "DTCUSTOAP"         => "DTCUSTOAP",
        "CDINDICE"          => "CDINDICE",
        "CDCONTROLE"        => "CDCONTROLE",
        "ONLYVIGSBG"        => "ONLYVIGSBG",
        "SERVCLASSE"        => "SERVCLASSE",
        "CDCICLO"           => "CDCICLO",
        "NRORG"             => "NRORG",
        "FILTERIT"          => "FILTERIT",
        "NRITAIRVISION"     => "NRITAIRVISION"
      ),
      'reportName'
    ),
    'COC03501' =>
    array (
      'strategy' => 'BIRT',
      'parameterMapping' =>
      array (
        "CDFILIAL"          => "CDFILIAL",
        "CDCIA"             => "CDCIA",
        "CDMANCIA"          => "CDMANCIA",
        "CDCLASSE"          => "CDCLASSE",
        "NRSERVICOCIAFILI"  => "NRSERVICOCIAFILI",
        "IDIOMA"            => "IDIOMA",
        "CDSUBGRU"          => "CDSUBGRU",
        "NMTIPOCUSTO"       => "IDTIPOCUSTO",
        "INFLACAO"          => "INFLACAO",
        "DTCUSTOAP"         => "DTCUSTOAP",
        "CDINDICE"          => "CDINDICE",
        "NRGERACAO"         => "NRGERACAO",
        "ONLYVIGSBG"        => "ONLYVIGSBG",
        "NRORG"             => "NRORG"
      ),
      'reportName'
    ),
    'COC03502' =>
    array (
      'strategy' => 'BIRT',
      'parameterMapping' =>
      array (
        "CDFILIAL"          => "CDFILIAL",
        "CDCIA"             => "CDCIA",
        "CDMANCIA"          => "CDMANCIA",
        "IDIOMA"            => "IDIOMA",
        "CDSUBGRU"          => "CDSUBGRU",
        "NMTIPOCUSTO"       => "IDTIPOCUSTO",
        "INFLACAO"          => "INFLACAO",
        "DTCUSTOAP"         => "DTCUSTOAP",
        "CDINDICE"          => "CDINDICE",
        "NRGERACAO"         => "NRGERACAO",
        "ONLYVIGSBG"        => "ONLYVIGSBG",
        "SERVCLASSE"        => "SERVCLASSE",
        "CDCICLO"           => "CDCICLO",
        "NRORG"             => "NRORG",
        "FILTERIT"          => "FILTERIT",
        "NRITAIRVISION"     => "NRITAIRVISION"
      ),
      'reportName'
    ),
    'COC03503' =>
    array (
      'strategy' => 'BIRT',
      'parameterMapping' =>
      array (
        "CDFILIAL"          => "CDFILIAL",
        "CDCIA"             => "CDCIA",
        "CDMANCIA"          => "CDMANCIA",
        "IDIOMA"            => "IDIOMA",
        "CDSUBGRU"          => "CDSUBGRU",
        "NMTIPOCUSTO"       => "IDTIPOCUSTO",
        "INFLACAO"          => "INFLACAO",
        "DTCUSTOAP"         => "DTCUSTOAP",
        "CDINDICE"          => "CDINDICE",
        "NRGERACAO"         => "NRGERACAO",
        "ONLYVIGSBG"        => "ONLYVIGSBG",
        "SERVCLASSE"        => "SERVCLASSE",
        "CDCICLO"           => "CDCICLO",
        "NRORG"             => "NRORG",
        "FILTERIT"          => "FILTERIT",
        "NRITAIRVISION"     => "NRITAIRVISION"
      ),
      'reportName'
    )
  );
  // prepara parametros do relatorio
  public function beforeProcessReport(DTO\Request\Row $request, DTO\Response $response) {
    $row = $request->getRow();
    $row['__report_name'] = $row['REPORTNAME'];
    if (isset($row['REPORTFORMAT']) && $row['REPORTFORMAT']){
      $this->birtStrategy->setReportFormat($row['REPORTFORMAT']);
    }
    $row['NRORG'] = $this->environment->getNrOrgTrab();
    if($row['REPORTNAME'] === "COC03500"){
      $prodManualciam = $this->prodManualciam($row, $response);
      $row['CDCONTROLE'] = $prodManualciam['P_CONTROLE'];
    }else{
      $geraAnaliseReceitam = $this->geraAnaliseReceitam($row, $response);
      $row['NRGERACAO'] = $geraAnaliseReceitam['P_NRGERACAO'];
    }
  }

  private function prodManualciam($row, $response) {
    try {
      $procedure = new StoredProcedure($this->connection, 'PROD_MANUALCIAM');
      $procedure->addParam(new Param('P_CDCIA', Param::PARAM_INPUT, $row['CDCIA']));
      $procedure->addParam(new Param('P_CDFILIAL', Param::PARAM_INPUT, $row['CDFILIAL']));
      $procedure->addParam(new Param('P_CDMANCIA', Param::PARAM_INPUT, $row['CDMANCIA']));
      $procedure->addParam(new Param('P_NRORG', Param::PARAM_INPUT, intval($this->environment->getNrOrgTrab())));
      $procedure->addParam(new Param('P_DTCUSTO', Param::PARAM_INPUT, $row['DTCUSTOAP']));
      $procedure->addParam(new Param('P_IDTIPOCUSTO', Param::PARAM_INPUT, intval($row['CDTIPOCUSTO'])));
      $procedure->addParam(new Param('P_CONTROLE', Param::PARAM_OUTPUT, null, null, 20));
      $controle = $procedure->execute();
      return $controle;
    } catch (\Exception $e) {
      $response->setError(new Error($e->getMessage(), 500));
    }
  }

  private function geraAnaliseReceitam($row, $response) {
    try {
      $procedure = new StoredProcedure($this->connection, 'GERA_ANALISERECEITAM');
      $procedure->addParam(new Param('P_CDFILIAL', Param::PARAM_INPUT, $row['CDFILIAL']));
      $procedure->addParam(new Param('P_NRGERACAO', Param::PARAM_OUTPUT, null, null, 50));
      $procedure->addParam(new Param('P_CDCIA', Param::PARAM_INPUT, $row['CDCIA']));
      $procedure->addParam(new Param('P_CDMANCIA', Param::PARAM_INPUT, $row['CDMANCIA']));
      $procedure->addParam(new Param('P_DTCUSTO', Param::PARAM_INPUT, $row['DTCUSTOAP']));
      $procedure->addParam(new Param('P_NRORG', Param::PARAM_INPUT, intval($this->environment->getNrOrgTrab())));
      $controle = $procedure->execute();
      return $controle;
    } catch (\Exception $e) {
      $response->setError(new Error($e->getMessage(), 500));
    }
  }
}