<?php

namespace Controllers;

use \Zeedhi\Framework\DTO;
use \Zeedhi\Framework\DTO\Response\Error;
use \Zeedhi\Framework\DataSource\Manager;
use \Zeedhi\Framework\DataSource\DataSet;
use \Zeedhi\Framework\DataSource\FilterCriteria;
use Zeedhi\Framework\DTO\Response\Message;
use \Zeedhi\Framework\DB\StoredProcedure\StoredProcedure;
use \Zeedhi\Framework\DB\StoredProcedure\Param;

class WorkflowController extends \Teknisa\GER\Controllers\TeknisaCrud{

	protected $dataSourceName;
	protected $dataSourceManager;
	protected $crudService;
	protected $environment;
	protected $parameterBagColumns = array();
	protected $service;
	protected $connection;
	protected $fileServerParams;
	protected $messagesProcedureInflightor;
	const WORKFLOW_APROVADO = 3;
	const WORKFLOW_REPROVADO = 2;

	public function __construct($crudService, $parameterbag, $dataSourceManager, $environment, $service, $connection, $fileServerParams, $messagesProcedureInflightor){
		$this->crudService = $crudService;
		$this->dataSourceManager = $dataSourceManager;
		$this->environment = $environment;
		$this->service = $service;
		$this->connection = $connection;
		$this->fileServerParams = $fileServerParams;
		$this->messagesProcedureInflightor = $messagesProcedureInflightor;
		parent::__construct($crudService, $parameterbag);
	}

	//metodo original
	public function processAprov(DTO\Request $request, DTO\Response $response){

		$this->dataSourceName = 'operworkflow';
		$row = $request->getRow();

		$filterOperWorkflow = array(
			array(
					"columnName" => "NRWORKFLOW",
					"operator" => "=",
					"value" => $row['NRWORKFLOW']
			),
			array(
					"columnName" => "CDOPERADOR",
					"operator" => "=",
					"value" => $this->environment->getCdOperador()
			)
		);

		$filterCriteria = new FilterCriteria($this->dataSourceName, $filterOperWorkflow);
		$filterCriteria =  $this->crudService->populateParameterBag($filterCriteria, $this->parameterBagColumns);
		$dataSet = $this->dataSourceManager->findBy($filterCriteria);
		$registros = $dataSet->getRows();

		$registros[0]["DTACAOWFLOW"] = $this->environment->getCurrentTime()->format("d/m/Y H:i:s");
		$registros[0]["IDSITOPERWFLOW"] = $row["IDSITOPERWFLOW"];
		$registros[0]["DSMOTIVOACAO"] = $row["DSMOTIVOACAO"];
		$dataSet = new DataSet($this->dataSourceName, $registros);
		$this->dataSourceManager->persist($dataSet);


		if($row['IDSITOPERWFLOW'] === 'N'){
			$this->aprovaWorkflow($request, $this->verificaOperEnvolvidos($row), $response);
		}else if($this->verificaOperEnvolvidos($row) === 'A'){
			$this->aprovaWorkflow($request, $this->verificaOperEnvolvidos($row), $response);
		}else{
			return $response->addMessage(new Message('COC00004_WORKFLOW_APROVADO'));
		}
	}

	public function verificaOperEnvolvidos($row){
		$pendencia = 'A';

		$filter = array(
			array(
					"columnName" => "NRWORKFLOW",
					"operator" => "=",
					"value" => $row['NRWORKFLOW']
			)
		);

		$filterCriteria = new FilterCriteria($this->dataSourceName, $filter);
		$filterCriteria =  $this->crudService->populateParameterBag($filterCriteria, $this->parameterBagColumns);
		$dataSet = $this->dataSourceManager->findBy($filterCriteria);
		$registros = $dataSet->getRows();
		foreach ($registros as $operador) {
			if($operador['IDSITOPERWFLOW'] === 'P'){
				$pendencia = 'N';
			}
		}
		return $pendencia;
	}

	public function aprovaWorkflow($request, $paramProcess, $response){
		$row = $request->getRow();

		if($row['IDSITOPERWFLOW'] === 'N'){

			$row['IDSITWORKFLOW'] = "2";
			$this->aplicaReprovaOper($row, $request, $response);

		}else if($paramProcess === "A" && $row['IDSITOPERWFLOW'] != 'N'){

			$row['IDSITWORKFLOW'] = "3";
			$rows = $this->crudService->getRowsToSave($request, $this->dataSourceName);
			$dataSet = new DataSet('workflow', $rows);
			$newRequest = new DTO\Request\DataSet($dataSet, '', '', '');
			$this->dataSourceManager->persist($dataSet);
			if($row['IDTIPOPERWFLOW'] === "3"){
				$this->deleteItemProcessoWorkFlow($row);
			}else{
				$this->updateItemProcessoWorkflow($row['NRWORKFLOW'], 'A');
			}
			return $response->addMessage(new Message("COC00004_WORKFLOW_APROVADO"));

		}else{
			$row['IDSITWORKFLOW'] = "1";
		}

	}

	public function aplicaReprovaOper($row, $request, $response){
		$rowsWorkflow = $this->crudService->getRowsToSave($request, $this->dataSourceName);
		$dataSetWorkflow = new DataSet('workflow', $rowsWorkflow);
		$this->dataSourceManager->persist($dataSetWorkflow);
		$this->updateItemProcessoWorkflow($row['NRWORKFLOW'], 'N');
		return $response->addMessage(new Message("COC00004_WORKFLOW_REPROVADO"));
	}

	public function varificaProcessosWorkflow($condition){

		$dataSourceDadoswflow = 'dadoswflow';

		$filter = array(
			array(
				"columnName"=> "DSPRIMARYKEY",
				"operator"=> "=",
				"value"=> $condition
			)
		);

		$filterCriteria = new FilterCriteria($dataSourceDadoswflow, $filter);
		$filterCriteria =  $this->crudService->populateParameterBag($filterCriteria, $this->parameterBagColumns);
		$dataSet = $this->dataSourceManager->findBy($filterCriteria);
		$registros = $dataSet->getRows();

		$workflows = array();

		$process = 'A';

		if(!empty($registros)){

			foreach ($registros as $row) {
				array_push($workflows, $row['NRWORKFLOW']);
			}

			$filterWorkflows = array(
				array(
					"columnName"=> "NRWORKFLOW",
					"operator"=> "IN",
					"value"=> $workflows
				)
			);

			$filterCriteriaWorkflows = new FilterCriteria("workflow", $filterWorkflows);
			$filterCriteriaWorkflows =  $this->crudService->populateParameterBag($filterCriteriaWorkflows, $this->parameterBagColumns);
			$dataSetWorkflows = $this->dataSourceManager->findBy($filterCriteriaWorkflows);
			$registrosWorkflows = $dataSetWorkflows->getRows();

			foreach ($registrosWorkflows as $workflow) {
				$process = $workflow['IDSITWORKFLOW'] === "2" ? "N": "A";
			}

		}

		return $process;

	}

	private function updateItemProcessoWorkflow($workflow, $aproved){
		$dataSourceDadoswflow = 'dadoswflow';
		$filter = array(
			array(
				'columnName' => 'NRWORKFLOW',
				'operator' => '=',
				'value' => $workflow
			)
		);
		$filterCriteria = new FilterCriteria($dataSourceDadoswflow, $filter);
		$filterCriteria =  $this->crudService->populateParameterBag($filterCriteria, $this->parameterBagColumns);
		$dataSet = $this->dataSourceManager->findBy($filterCriteria);
		$registros = $dataSet->getRows();


		$itemDataSource = $registros[0]['NMTABELAWFLOW'];

		$filterItem = array();

		foreach ($registros as $row) {
        $parameters = explode(';', $row['DSPRIMARYKEY']);
	        foreach ($parameters as $parameter) {
	        	if(strlen($parameter) != 0){
		            $keyAndValue = explode('=', $parameter);
		            array_push($filterItem, array(
		            		'columnName' => $keyAndValue[0],
		            		'operator' => '=',
		            		'value' => $keyAndValue[1]
		            	)
		            );
	        	}
	    	}
	    }

    	$filterCriteriaItem = new FilterCriteria(strtolower($itemDataSource), $filterItem);
		$filterCriteriaItem =  $this->crudService->populateParameterBag($filterCriteriaItem, $this->parameterBagColumns);
		$dataSetItem = $this->dataSourceManager->findBy($filterCriteriaItem);
		$registrosItem = $dataSetItem->getRows();
		$itensToUpdate = array();

		if($itemDataSource == 'SUBRECEITAM' && $aproved === 'N' || $itemDataSource == 'PRORECEITAM'  && $aproved === 'N'){

			$registersToDelete = array();

			foreach ($registros as $row) {
		        $parameters = explode(';', $row['DSPRIMARYKEY']);
		        foreach ($parameters as $parameter) {
					if(strlen($parameter) != 0){
						$keyAndValue = explode('=', $parameter);
						$rowToDelete[$keyAndValue[0]] = $keyAndValue[1];
					}
		        }
	    		$rowToDelete['__is_new'] = false;
	        	array_push($registersToDelete, $rowToDelete);
	    	}

			$dataSetDeleteItem = new DataSet(strtolower($itemDataSource), (array)$registersToDelete);
	    	$this->dataSourceManager->delete($dataSetDeleteItem);

		}else{
			foreach ($registrosItem as $item) {
				$item[$registros[0]['NMCAMPOWFLOW']] = $aproved === 'A' ? $registros[0]['DSINFORATUAL'] : $registros[0]['DSINFORANTERIOR'];
				$item['DTULTATU'] = $this->environment->getCurrentTime()->format("d/m/Y H:i:s");
				$item['IDANALIZEATIV'] = $this->varificaProcessosWorkflow($registros[0]['DSPRIMARYKEY']);
				$item['changed'] = true;
				$item['__is_new'] = false;
				array_push($itensToUpdate, $item);
			}

		}


		$dataSetUpdateItem = new DataSet(strtolower($itemDataSource), $itensToUpdate);
		$this->dataSourceManager->persist($dataSetUpdateItem);
	}

	public function deleteItemProcessoWorkFlow($row){
		$dataSourceDadoswflow = 'dadoswflow';
		$filter = array(
			array(
				'columnName' => 'NRWORKFLOW',
				'operator' => '=',
				'value' => $row['NRWORKFLOW']
			)
		);
		$filterCriteria = new FilterCriteria($dataSourceDadoswflow, $filter);
		$filterCriteria =  $this->crudService->populateParameterBag($filterCriteria, $this->parameterBagColumns);
		$dataSet = $this->dataSourceManager->findBy($filterCriteria);
		$registros = $dataSet->getRows();

		$itemDataSource = $registros[0]['NMTABELAWFLOW'];

		$registersToDelete = array();

		foreach ($registros as $row) {
        $parameters = explode(';', $row['DSPRIMARYKEY']);
        foreach ($parameters as $parameter) {
          $keyAndValue = explode('=', $parameter);
        	$rowToDelete[$keyAndValue[0]] = $keyAndValue[1];
        }
    		$rowToDelete['__is_new'] = false;
        array_push($registersToDelete, $rowToDelete);
    }

    $dataSetDeleteItem = new DataSet(strtolower($itemDataSource), (array)$registersToDelete);
    $this->dataSourceManager->delete($dataSetDeleteItem);
	}

//-------------------- nova implementação da execução do workflow
	public function aprovaWflow($request, $response){
		$row = (array)$request->getRow();
		$this->connection->beginTransaction();
		try{
			$msg = '';
			$conditionsOperWorkflow = array(
				array(
					'columnName' => 'NRWORKFLOW',
					'operator' => '=',
					'value' => $row['NRWORKFLOW']
				),
				array(
					'columnName' => 'NRORG',
					'operator' => '=',
					'value' => $this->environment->getNrOrg()
				),
				array(
					'columnName' => 'CDOPERADOR',
					'operator' => '<>',
					'value' => $this->environment->getCdOperador()
				),
				array(
					'columnName' => 'IDSITOPERWFLOW',
					'operator' => 'IN',
					'value' => array('P','N')
				)
			);

			if ($row['NRPROCESSOAPROVA'] == 4 && in_array($row['NRITEMPROCAPROVA'], array(26, 27))) { // IMPORTACAO DE SERVICO + SUBSTITUICAO DE COMPOSICAO
				if($row['IDSITOPERWFLOW'] == 'A' && $this->service->canExecuteWorkflowOperation($conditionsOperWorkflow)){
						$metodoDoProcesso = $row['NRITEMPROCAPROVA'] == 26 ? $this->executaImportacaoServico($row, $response) : $this->executaSubstituiComposicao($row, $response);
						if ($metodoDoProcesso) {
							$this->service->changeWorkFlowStatus($row, 3);
							$this->service->changeOperWorkflowStatus($row);
							$response->addMessage(new Message('COC00004_WORKFLOW_APROVADO'));
							$this->connection->commit();
						} else {
							$this->connection->rollback();
						}
				} else {
					$msg = "";
					switch($row['IDSITOPERWFLOW']){
						case 'A':
							$this->service->changeWorkFlowStatus($row, 3);
							$msg = 'COC00004_WORKFLOW_APROVADO'; break;
						case 'N':
							$this->service->changeWorkFlowStatus($row, 2);
							$this->removeServicosWorkflowCpSvMenu($row, $response);
							$msg = 'COC00004_WORKFLOW_REPROVADO'; break;
						case 'C':
							$msg = 'COC00004_OPER_CIENTE_WORKFLOW'; break;
					}
					$this->service->changeOperWorkflowStatus($row);
					$response->addMessage(new Message($msg));
				}
			} else {
				$dataError = array();
				if($row['IDSITOPERWFLOW'] == 'A' && $this->service->canExecuteWorkflowOperation($conditionsOperWorkflow)){
					$dataWorkFlow = $this->service->getDataToExecuteOperation($row['NRWORKFLOW']);
					$dataError = $this->service->executeOperation($dataWorkFlow['rowOperation'], $dataWorkFlow['primaryKeys'], $dataWorkFlow['dataSave']);
					$this->service->changeWorkFlowStatus($row, 3);
				} else if($row['IDSITOPERWFLOW'] == 'N'){
					if($row['NRPROCESSOAPROVA'] === 5 && $row['NRITEMPROCAPROVA'] === 1){
						//Reprova a alteração de Finalização do Desenvolvimento da Versao  Menu Grid Versionamento
						$this->service->changeStatusVersao($row);
					}
					$this->service->changeWorkFlowStatus($row, 2);
				}
				$this->service->changeOperWorkflowStatus($row);
				
				if(empty($dataError)){
					switch($row['IDSITOPERWFLOW']){
						case 'A':
							$msg = 'COC00004_WORKFLOW_APROVADO'; break;
						case 'N':
							$msg = 'COC00004_WORKFLOW_REPROVADO'; break;
						case 'C':
							$msg = 'COC00004_OPER_CIENTE_WORKFLOW'; break;
					}
					$response->addMessage(new Message($msg));
					$this->connection->commit();
				} else {
					$this->connection->rollback();
					$response->addDataSet(new DataSet('errorMessage', $dataError));
				}
			}
		} catch (\Exception $e){
			$response->setError(new Error($this->messagesProcedureInflightor->cleanMessage($e->getMessage()), 1));
			$this->connection->rollback();
		}
	}
	
	private function executaImportacaoServico($row, $response) {
        $procedure = new StoredProcedure($this->connection, 'COPIA_SERV_MENU_GRID_APROVADO');
        $procedure->addParam(new Param('P_CDFILIPROC', Param::PARAM_INPUT, $this->environment->getCdFilial(), Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param('P_NRWORKFLOW', Param::PARAM_INPUT, $row['NRWORKFLOW'], Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param('P_NRORG', Param::PARAM_INPUT, $this->environment->getNrOrg(), Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param('P_CDOPERADOR', Param::PARAM_INPUT, $this->environment->getCdOperador(), Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param('P_IMAGEMDEFAULT', Param::PARAM_INPUT, $this->fileServerParams->getUndefinedImagePath(), Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param('P_NRLISTAERROS', Param::PARAM_OUTPUT, null, null, 20));
        $resultado = $procedure->execute();

        if($resultado['P_NRLISTAERROS'] != null) {
        	$this->messagesProcedureInflightor->getErrorMessages($row['CDFILIAL'], $resultado['P_NRLISTAERROS'], $response);
        	return false;
        } else {
        	return true;
        }
	}
	
	private function removeServicosWorkflowCpSvMenu($row, $response) {
		$query = "DELETE FROM WORKFLOWCPSERVMENU WHERE NRWORKFLOW = :NRWORKFLOW AND NRORG = :NRORG";
		$params = array(
			"NRWORKFLOW" => $row["NRWORKFLOW"],
			"NRORG" => $this->environment->getNrOrg()
		);
		$stmt = $this->connection->prepare($query);
		$stmt->execute($params);
	}
	
	private function executaSubstituiComposicao($row, $response) {
		$nrsubscompmenu = $this->getNrsubscompmenu($row);
		$filialLogada = $this->environment->getCdFilial();
		$nrorg = $this->environment->getNrOrgTrab();
		$cdoperador = $this->environment->getCdOperador();
		
        $procedure = new StoredProcedure($this->connection, 'APROVA_SUBSCOMPMENU');
        
        $procedure->addParam(new Param('P_NRSUBSCOMPMENU', Param::PARAM_INPUT, $nrsubscompmenu, Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param('P_CDFILIAL', Param::PARAM_INPUT, $filialLogada, Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param('P_NRORG', Param::PARAM_INPUT, $nrorg, Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param('P_CDOPERADOR', Param::PARAM_INPUT, $cdoperador, Param::PARAM_TYPE_STR));
        $resultado = $procedure->execute();
        
		return true;
	}
	
	private function getNrsubscompmenu($row) {
		$query = 'SELECT DSPRIMARYKEY FROM DADOSWFLOW WHERE NRWORKFLOW = :NRWORKFLOW AND NRORG = :NRORG';
		$stmt = $this->connection->prepare($query);
		$params = array(
			'NRWORKFLOW' => $row['NRWORKFLOW'],
			'NRORG' => $this->environment->getNrOrgTrab()
		);
		
		$stmt->execute($params);
		
		$result = $stmt->fetchAll(\PDO::FETCH_ASSOC);
		
		if (sizeof($result) != 1) {
			return false;
		} else {
			return $result[0]['DSPRIMARYKEY'];
		}
	}
	
	public function processAprovWFUnidade(DTO\Request $request, DTO\Response $response){
		$row = (array)$request->getRow();
		$this->connection->beginTransaction();
		
		try{
			$this->updateOperadoresEnvolvidos($row);
			$this->updateWorkflowExclusaoUnidade($row);
			$this->updateDadosWorkflowUnidade($row, $response);
			$this->connection->commit();
			$response->addMessage(new Message('COC00004_WORKFLOW_APROVADO'));
		} catch (\Exception $e){
			$response->setError(new Error($this->messagesProcedureInflightor->cleanMessage($e->getMessage()), 1));
			$this->connection->rollback();
		}
		
	}
	
	private function updateOperadoresEnvolvidos($row){
		
		$sql = "UPDATE OPERWORKFLOW
                   SET IDSITOPERWFLOW = :V_IDSITOPERWFLOW,
                       DTACAOWFLOW    = SYSDATE,
                       DSMOTIVOACAO   = :V_DSMOTIVOACAO,
                       DTULTATU       = SYSDATE,
                       NRORGULTATU    = :V_NRORGULTATU,
                       CDOPERULTATU   = :V_CDOPERULTATU
                 WHERE NRWORKFLOW     = :P_NRWORKFLOW
                   AND CDOPERADOR     = :P_CDOPERADOR
                   AND NRORG          = :P_NRORG";
                   
        $params = array(
        	"V_IDSITOPERWFLOW" => $row["IDSITOPERWFLOW"],      
			"V_DSMOTIVOACAO"   => $row["DSMOTIVOACAO"],    
			"V_NRORGULTATU"    => $this->environment->getNrOrgTrab(),   
			"V_CDOPERULTATU"   => $this->environment->getCdOperador(),    
			"P_NRWORKFLOW"     => $row["NRWORKFLOW"],  
			"P_CDOPERADOR"     => $this->environment->getCdOperador(),  
			"P_NRORG"          => $this->environment->getNrOrgTrab()
		);
		
		$stmt = $this->connection->prepare($sql);
		$stmt->execute($params);
		
	}
	
	private function updateWorkflowExclusaoUnidade($row){
		
		$sql = "UPDATE WORKFLOW
                   SET IDSITWORKFLOW  = :V_IDSITWORKFLOW,
                       DTULTATU       = SYSDATE,
                       NRORGULTATU    = :V_NRORGULTATU,
                       CDOPERULTATU   = :V_CDOPERULTATU
                 WHERE NRWORKFLOW     = :P_NRWORKFLOW
                   AND NRORG          = :P_NRORG";

        $params = array(
        	"V_IDSITWORKFLOW"  => $row["IDSITOPERWFLOW"] == 'N'? self::WORKFLOW_REPROVADO : self::WORKFLOW_APROVADO,
			"V_NRORGULTATU"    => $this->environment->getNrOrgTrab(),
			"V_CDOPERULTATU"   => $this->environment->getCdOperador(),
			"P_NRWORKFLOW"     => $row["NRWORKFLOW"],
			"P_NRORG"          => $this->environment->getNrOrgTrab()
		);
		
		$stmt = $this->connection->prepare($sql);
		$stmt->execute($params);
		
	}
	
	private function updateDadosWorkflowUnidade($row, $response){
		if($row['IDSITOPERWFLOW'] == 'A'){
			
			$dadosWflow = $this->retornaDadosWorkflow($row);
			
			$condicao = explode(';', $dadosWflow[0]['DSPRIMARYKEY']);
			$condicao = implode(' AND ', $condicao);
			
			$sql = "UPDATE ".$dadosWflow[0]['NMTABELAWFLOW']." 
			           SET NRVARIACAOREC = :V_NRVARIACAOREC, DTULTATU = SYSDATE, NRORGULTATU = :V_NRORGULTATU, CDOPERULTATU = :V_CDOPERULTATU 
			         WHERE ".$condicao;
			         
			$params = array(
	        	"V_NRVARIACAOREC" => $dadosWflow[0]['DSINFORATUAL'],
				"V_NRORGULTATU"   => $this->environment->getNrOrgTrab(),
				"V_CDOPERULTATU"  => $this->environment->getCdOperador()
			);
			
			$stmt = $this->connection->prepare($sql);
			$stmt->execute($params);
			
		}else{
			$response->addMessage(new Message('COC00004_WORKFLOW_N_APROVADO'));
		}
		
	}
	
	private function retornaDadosWorkflow($row){
		
		$sql = "SELECT NRWORKFLOW, NMTABELAWFLOW, NMCAMPOWFLOW,
                       DSINFORANTERIOR, DSINFORATUAL, DSPRIMARYKEY,    
                       NRORG, DTINCLUSAO, CDOPERINCLUSAO,  
                       DTULTATU, NRORGINCLUSAO, NRORGULTATU,     
                       CDOPERULTATU, IDATIVO         
                  FROM DADOSWFLOW
                 WHERE NRWORKFLOW = :P_NRWORKFLOW
                   AND NRORG      = :P_NRORG";
                   
        $params = array(  
			"P_NRWORKFLOW" => $row["NRWORKFLOW"], 
			"P_NRORG"      => $this->environment->getNrOrgTrab()
		);
		
		$result = $this->connection->fetchAll($sql, $params);
		
		return $result;
		
	}
	
	private function populaWorkflowProcEItems(){
		$cdoperador = $this->environment->getCdOperador();
		$nrorg = $this->environment->getNrOrgTrab();
		$procedure = new StoredProcedure($this->connection, 'DEFINICOES_WORKFLOW');
        $procedure->addParam(new Param('P_CDOPERADOR', Param::PARAM_INPUT, $cdoperador, Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param('P_NRORG', Param::PARAM_INPUT, $nrorg, Param::PARAM_TYPE_STR));
        $resultado = $procedure->execute();
	}
	
	public function getWorkflowProcEItems(DTO\Request $request, DTO\Response $response){
		$this->populaWorkflowProcEItems();
		
		$sql = "
		SELECT '' AS PARENT,
			   0 AS NRORDER,
			   TO_CHAR(NRPROCESSOAPROVA) AS PRIMARY,
			   'PROCESSOAPROVA_' || TO_CHAR(NRPROCESSOAPROVA) AS NAME,
			   0 AS NRDIASPROCAPROVA,
			   IDATIVO,
			   NRORG
		  FROM PROCESSOAPROVA
		 UNION
		SELECT TO_CHAR(NRPROCESSOAPROVA) AS PARENT,
		       NRITEMPROCAPROVA AS NRORDER,
		       'p' || TO_CHAR(NRITEMPROCAPROVA) AS PRIMARY,
		       'ITEMPROCAPROVA_'|| TO_CHAR(NRPROCESSOAPROVA) || '_' || TO_CHAR(NRITEMPROCAPROVA) AS NAME,
		       NRDIASPROCAPROVA,
		       IDATIVO,
		       NRORG
		  FROM ITEMPROCAPROVA
		 ORDER BY NRORDER
		";
		$query = $this->connection->prepare($sql);
		$query->execute();
		$newDataSet = new DataSet('getWorkflowProcEItems', $query->fetchAll());
		$response->addDataSet($newDataSet);
	}
	
	public function vinculaOperItems(DTO\Request $request, DTO\Response $response){
		$rows = $request->getDataSet()->getRows();
		foreach($rows as $row){
			$cdoperador = $this->environment->getCdOperador();
			$nrorg = $this->environment->getNrOrgTrab();
			
			$sql = "
			INSERT INTO OPERITEMPAPROVA
			(NRPROCESSOAPROVA, NRITEMPROCAPROVA, CDOPERADOR,
			 NRDIASOPERAPRO, IDTIPOACAOOPER, NRPTOACAOOPER,
			 NRORG, NRORGINCLUSAO, NRORGULTATU,
			 CDOPERINCLUSAO, CDOPERULTATU,
			 DTINCLUSAO, DTULTATU)
			SELECT :NRPROCESSOAPROVA, :NRITEMPROCAPROVA, :CDOPERADOR,
			       :NRDIASOPERAPRO, :IDTIPOACAOOPER, :NRPTOACAOOPER,
			       :NRORG, :NRORG, :NRORG,
			       :CDOPERINC, :CDOPERINC,
			       SYSDATE, SYSDATE
			  FROM DUAL
			 WHERE NOT EXISTS (SELECT 1
			                     FROM OPERITEMPAPROVA OA
			                    WHERE OA.CDOPERADOR = :CDOPERADOR
			                      AND OA.NRPROCESSOAPROVA = :NRPROCESSOAPROVA
			                      AND OA.NRITEMPROCAPROVA = :NRITEMPROCAPROVA)
			";
			$params = [
				'NRPROCESSOAPROVA' => $row['NRPROCESSOAPROVA'],
				'NRITEMPROCAPROVA' => $row['NRITEMPROCAPROVA'],
				'CDOPERADOR' =>  $row['CDOPER'],
			    'NRDIASOPERAPRO' =>  $row['NRDIASOPERAPRO'],
			    'IDTIPOACAOOPER' =>  $row['IDTIPOACAOOPER'],
			    'NRPTOACAOOPER' =>  $row['NRPTOACAOOPER'],
				'NRORG' => $nrorg,
				'CDOPERINC' => $cdoperador
			];
			$command = $this->connection->prepare($sql);
			$command->execute($params);	
		}
		
		
	}
	
	public function desvinculaOperItems(DTO\Request $request, DTO\Response $response){
		$rows = $request->getDataSet()->getRows();
		foreach($rows as $row){
			$sql = "
			DELETE FROM OPERITEMPAPROVA
			 WHERE NRPROCESSOAPROVA = :NRPROCESSOAPROVA
			   AND NRITEMPROCAPROVA = :NRITEMPROCAPROVA
			   AND CDOPERADOR = :CDOPERADOR
			";
			$params = [
				'NRPROCESSOAPROVA' => $row['NRPROCESSOAPROVA'],
				'NRITEMPROCAPROVA' => $row['NRITEMPROCAPROVA'],
				'CDOPERADOR' =>  $row['CDOPER']	
			];
			$command = $this->connection->prepare($sql);
			$command->execute($params);
		}
	}
	
	public function atualizaOperItem(DTO\Request $request, DTO\Response $response){
		$row = $request->getRow();
		
		$cdoperador = $this->environment->getCdOperador();
		$nrorg = $this->environment->getNrOrgTrab();
		
		$sql = "UPDATE OPERITEMPAPROVA
		           SET NRDIASOPERAPRO = :NRDIASOPERAPRO,
		               IDTIPOACAOOPER = :IDTIPOACAOOPER,
		               NRORGULTATU = :NRORG,
		               CDOPERULTATU = :CDOPERATU,
		               DTULTATU = SYSDATE
		         WHERE NRPROCESSOAPROVA = :NRPROCESSOAPROVA
		           AND NRITEMPROCAPROVA = :NRITEMPROCAPROVA
		           AND CDOPERADOR = :CDOPERADOR";
		$params = [
				'NRPROCESSOAPROVA' => $row['NRPROCESSOAPROVA'],
				'NRITEMPROCAPROVA' => $row['NRITEMPROCAPROVA'],
				'CDOPERADOR' =>  $row['CDOPER'],
			    'NRDIASOPERAPRO' =>  $row['NRDIASOPERAPRO'],
			    'IDTIPOACAOOPER' =>  $row['IDTIPOACAOOPER'],
				'NRORG' => $nrorg,
				'CDOPERATU' => $cdoperador
			];
		$command = $this->connection->prepare($sql);
		$command->execute($params);
	}
}

