<?php

namespace Services;

use Teknisa\GER\Utilities\Environment;
use Teknisa\GER\Services\CrudService;
use Zeedhi\Framework\DataSource\DataSet;
use Zeedhi\Framework\DataSource\FilterCriteria;
use Zeedhi\Framework\DataSource\Manager;
use Zeedhi\Framework\DataSource\ParameterBag;
use Zeedhi\Framework\DTO;
use Zeedhi\Framework\DTO\Response\Error;
use Zeedhi\Framework\DTO\Response\Message;
use Zeedhi\Framework\DB\StoredProcedure\Param;
use Zeedhi\Framework\DB\StoredProcedure\StoredProcedure;
use Zeedhi\Framework\Remote\FileUploader\Server;

class Coc00022ManutencaoMenugridService{

	protected $connection;
	protected $crudService;
	protected $workflowService;

	public function __construct($connection, $crudService, $workflowService) {
		$this->connection = $connection;
		$this->crudService = $crudService;
		$this->workflowService = $workflowService;
	}

	protected function execute($sql, $params){
		$query = $this->connection->prepare($sql);
		$query->execute($params);
	}

	public function deleteLastProporcao($row){
		$sql = "DELETE FROM AUXPROPSUBGRUSE
		WHERE CDFILIAL = :CDFILIAL
		AND CDCIA = :CDCIA
		AND CDMANCIA = :CDMANCIA
		AND NRSERVICOCIAFILI = :NRSERVICOCIAFILI
		AND CDCLASSE = :CDCLASSE
		AND CDSUBGRU = :CDSUBGRU
		AND NRSEQSUBGRUSEMAM = :NRSEQSUBGRUSEMAM
		AND NRORG = :NRORG
		AND QTPAXDE >= ALL(SELECT QTPAXDE FROM AUXPROPSUBGRUSE
			WHERE CDFILIAL = :CDFILIAL
			AND CDCIA = :CDCIA
			AND CDMANCIA = :CDMANCIA
			AND NRSERVICOCIAFILI = :NRSERVICOCIAFILI
			AND CDCLASSE = :CDCLASSE
			AND CDSUBGRU = :CDSUBGRU
			AND NRSEQSUBGRUSEMAM = :NRSEQSUBGRUSEMAM
		)";
		$params = array(
			'CDFILIAL' => $row['CDFILIAL'],
			'CDCIA' => $row['CDCIA'],
			'CDMANCIA' => $row['CDMANCIA'],
			'NRSERVICOCIAFILI' => $row['NRSERVICOCIAFILI'],
			'CDCLASSE' => $row['CDCLASSE'],
			'CDSUBGRU' => $row['CDSUBGRU'],
			'NRSEQSUBGRUSEMAM' => $row['NRSEQSUBGRUSEMAM'],
			'NRORG' => $row['NRORG']
		);

		$this->execute($sql, $params);
	}

    private function retornaRegistroComFoto($fgfotosubsemam) {
        $stmt = $this->connection->prepare('SELECT COUNT(1) AS QTDE FROM CICLOSUBGRUSEMAM WHERE FGFOTOSUBSEMAM = :FGFOTOSUBSEMAM');
        $stmt->execute(array('FGFOTOSUBSEMAM' => $fgfotosubsemam));
        $result = $stmt->fetch(\PDO::FETCH_ASSOC);
        return $result['QTDE'];
    }

    private function retornaRegistroComFotoAux($fgfotosubsemam) {
        $stmt = $this->connection->prepare('SELECT COUNT(1) AS QTDE FROM AUXCICLOSUBGRUSEMAM WHERE FGFOTOSUBSEMAM = :FGFOTOSUBSEMAM');
        $stmt->execute(array('FGFOTOSUBSEMAM' => $fgfotosubsemam));
        $result = $stmt->fetch(\PDO::FETCH_ASSOC);
        return $result['QTDE'];
    }

    public function tryDeleteImage($row){
        $imgInCiclosubsemam = $this->retornaRegistroComFoto($row['OLD_FGFOTOSUBSEMAM']);
        $imgInCiclosubsemamt = $this->retornaRegistroComFoto($row['OLD_FGFOTOSUBSEMAMT']);
        $imgInAuxcicloSubsemam = $this->retornaRegistroComFotoAux($row['OLD_FGFOTOSUBSEMAM']);
        $imgInAuxcicloSubsemamt = $this->retornaRegistroComFotoAux($row['OLD_FGFOTOSUBSEMAMT']);
        if($imgInAuxcicloSubsemam <= 1 && $imgInCiclosubsemam == 0){
                $this->deleteImageInServer($row['OLD_FGFOTOSUBSEMAM'],$this->crudService->getEnvironment()->getNrOrg());
        }
        if($imgInAuxcicloSubsemamt <= 1 && $imgInCiclosubsemamt == 0){
                $this->deleteImageInServer($row['OLD_FGFOTOSUBSEMAMT'],$this->crudService->getEnvironment()->getNrOrg());
        }
    }

	private function deleteImageInServer($flieURL, $nrorg){

		//@todo - refatorar de acordo com a regra de negócio do método tryDeleteImage

		$stmt = $this->connection->prepare("SELECT COUNT(1) AS GARBAGE FROM FSGARBAGE WHERE FILEURL = :FILEURL AND NRORG = :NRORG");

		$params = array(
			"FILEURL" => $flieURL,
			"NRORG" => $nrorg
		);

		$stmt->execute($params);

		$result = $stmt->fetch(\PDO::FETCH_ASSOC);

		if($result['GARBAGE'] == 0) {

			$garbage = $this->connection->prepare("INSERT INTO FSGARBAGE (FILEURL, NRORG) VALUES (:FILEURL, :NRORG)");
			$garbage->execute($params);

		}

	}

	public function executeOperationCiclo($tableName,$row){
		$params = array(
			'CDFILIAL' => $row['CDFILIAL'],
			'CDCIA' => $row['CDCIA'],
			'CDMANCIA' => $row['CDMANCIA'],
			'NRSERVICOCIAFILI' => $row['NRSERVICOCIAFILI'],
			'CDCLASSE' => $row['CDCLASSE'],
			'CDSUBGRU' => $row['CDSUBGRU'],
			'NRSEQSUBGRUSEMAM' => $row['NRSEQSUBGRUSEMAM'],
			'CDCICLO' => $row['CDCICLO']
		);

		$sql = "DELETE FROM ".$tableName." WHERE CDFILIAL = :CDFILIAL
								AND CDCIA = :CDCIA
								AND CDMANCIA = :CDMANCIA
								AND NRSERVICOCIAFILI = :NRSERVICOCIAFILI
								AND CDCLASSE = :CDCLASSE
								AND CDSUBGRU = :CDSUBGRU
								AND NRSEQSUBGRUSEMAM = :NRSEQSUBGRUSEMAM
								AND CDCICLO = :CDCICLO";
		$this->execute($sql, $params);
	}

	public function getParamsToExecuteOperations($row){
		$params = array(
			'CDFILIAL' => $row['CDFILIAL'],
			'CDCIA' => $row['CDCIA'],
			'CDMANCIA' => $row['CDMANCIA'],
			'NRSERVICOCIAFILI' => $row['NRSERVICOCIAFILI'],
			'CDCLASSE' => $row['CDCLASSE'],
			'CDSUBGRU' => $row['CDSUBGRU'],
			'NRSEQSUBGRUSEMAM' => $row['NRSEQSUBGRUSEMAM']
		);

		return $params;
	}

	public function deleteSonDataSubgrupo($params){
		$tablesToDelete = array('AUXPRODSUBSEMAM', 'AUXRECSUBSEMAM', 'AUXSUBGRUPRINCIPALMENU',
			'AUXMONTASUBSEMAM', 'AUXCICLOSUBGRUSEMAM', 'AUXPROCIASUBSEMAM', 'AUXPROPSUBGRUSE');

		foreach($tablesToDelete as $tableName){
			$sql = "DELETE FROM ".$tableName." WHERE CDFILIAL = :CDFILIAL
									AND CDCIA = :CDCIA
									AND CDMANCIA = :CDMANCIA
									AND NRSERVICOCIAFILI = :NRSERVICOCIAFILI
									AND CDCLASSE = :CDCLASSE
									AND CDSUBGRU = :CDSUBGRU
									AND NRSEQSUBGRUSEMAM = :NRSEQSUBGRUSEMAM";
			$this->execute($sql, $params);
		}
	}

	public function deleteImagesCiclos($rows, $defaultImage){
		foreach($rows as $ciclo){
			$ciclo['OLD_FGFOTOSUBSEMAM'] = $ciclo['FGFOTOSUBSEMAM'];
			$ciclo['OLD_FGFOTOSUBSEMAMT'] = $ciclo['FGFOTOSUBSEMAMT'];
			if($ciclo['OLD_FGFOTOSUBSEMAM'] != $defaultImage && !empty($ciclo['OLD_FGFOTOSUBSEMAM'])){
				$this->tryDeleteImage($ciclo);
			}
		}
	}

	public function executeDuplicateProcedure($row){
		$environment = $this->crudService->getEnvironment();

		$procedure = new StoredProcedure($this->connection, 'DUPLICA_SUBGRUPO_MENUGRID');
		$procedure->addParam(new Param('CDFILIAL',         Param::PARAM_INPUT, $row['CDFILIAL']));
		$procedure->addParam(new Param('CDCIA',            Param::PARAM_INPUT, $row['CDCIA']));
		$procedure->addParam(new Param('CDMANCIA',         Param::PARAM_INPUT, $row['CDMANCIA']));
		$procedure->addParam(new Param('NRSERVICOCIAFILI', Param::PARAM_INPUT, $row['NRSERVICOCIAFILI']));
		$procedure->addParam(new Param('CDCLASSE',         Param::PARAM_INPUT, $row['CDCLASSE']));
		$procedure->addParam(new Param('CDSUBGRU',         Param::PARAM_INPUT, $row['CDSUBGRU']));
		$procedure->addParam(new Param('NRSEQSUBGRUSEMAM', Param::PARAM_INPUT, $row['NRSEQSUBGRUSEMAM']));
		$procedure->addParam(new Param('CDOPERADOR',       Param::PARAM_INPUT, $environment->getCdOperador()));
		$procedure->addParam(new Param('NRORG',            Param::PARAM_INPUT, $environment->getNrOrg()));

		$procedure->execute();
	}

	public function executeEfetivaSubgrupo($row){
		$environment = $this->crudService->getEnvironment();

		$procedure = new StoredProcedure($this->connection, 'EFETIVA_SUBGRU_DUPLICADO');
		$procedure->addParam(new Param('CDFILIAL',         Param::PARAM_INPUT, $row['CDFILIAL']));
		$procedure->addParam(new Param('CDCIA',            Param::PARAM_INPUT, $row['CDCIA']));
		$procedure->addParam(new Param('CDMANCIA',         Param::PARAM_INPUT, $row['CDMANCIA']));
		$procedure->addParam(new Param('NRSERVICOCIAFILI', Param::PARAM_INPUT, $row['NRSERVICOCIAFILI']));
		$procedure->addParam(new Param('CDCLASSE',         Param::PARAM_INPUT, $row['CDCLASSE']));
		$procedure->addParam(new Param('CDSUBGRU',         Param::PARAM_INPUT, $row['CDSUBGRU']));
		$procedure->addParam(new Param('NRSEQSUBGRUSEMAM', Param::PARAM_INPUT, $row['NRSEQSUBGRUSEMAM']));
		$procedure->addParam(new Param('CDOPERADOR',       Param::PARAM_INPUT, $environment->getCdOperador()));
		$procedure->addParam(new Param('NRORG',            Param::PARAM_INPUT, $environment->getNrOrg()));

		$procedure->execute();
	}

	public function executeWorkflow($row){
		$workflowRow = $this->workflowService->createRowToWorkFlowProcess(array('IDANALIZEATIV' => 'N/A'), $row['PK'], $row['DSMOTIVOACAO'], $row['NMITEM'], 'SUBGRUSEMAM', 2, 24, 4);
		$this->workflowService->createWorkFlowProcess($row['PK'], array(), $workflowRow);
	}
}